class: inverse
layout: true
name: Introduction to IT for new employees

---
class: title, center, middle

# Bare metal, shells and clouds ​- IT2Research​

![](./img/logo_imgw.svg)
![](../img/logo_fgga2.svg)


9.10.2023 - M. Blaschek - PhD Seminar @ IMGW

---

# What can you expect from this presentation?

- Asking Questions (anytime, please interrupt)
- Hardware 
- Data and Storage
- Services
- Rules
- Updates



---

# Important Things @UNIVIE

The ZID offers a lot of information on its services, most importantly your **u:account**.

Services:

- [u:account](https://zid.univie.ac.at/uaccount/) everything depends on it
- [vpn](https://zid.univie.ac.at/vpn/) access to only university wide things (our servers)
- [u:cloud](https://ucloud.univie.ac.at) University cloud
- [u:access](https://bibliothek.univie.ac.at/en/uaccess.html) library access for publications
- [gitlab/mattermost](https://gitlab.phaidra.org/imgw) code development / socializing
- [u:wiki](https://wiki.univie.ac.at/display/theomet) documentation of workflows, group communication, tutorials, intranet, ...
    * Lots of tutorials for specific problems (travel, expenses, new employees, printer, ...)
    * HPC / Computing information and guidelines
    * Please login once. More access can be granted afterwards.
- [wolke.img.univie.ac.at](https://wolke.img.univie.ac.at)
    * Landing Page of IT service @ IMGW
    * User Management [IPA](https://wolke.img.univie.ac.at/ipa/ui)
---

# Hardware @ IMGW

Documentation: [Computing@u:wiki](https://wiki.univie.ac.at/x/6kW0Bw) or [Computing@gitlab](https://gitlab.phaidra.org/imgw/computer-resources) (public)

Currently the department has:
- SRVX1 (Teaching and development node) access: open
    * All Students
    * All Staff
    * Teaching
    * Development Node for interactive computing
- SRVX8 (Staff, graphical node, VNC, GPU) access: univie
    * special use cases
    * used for VNC
- JET Cluster (2+7, Staff, HPC) access: univie
    * default access for staff
    * some master students
    * slurm scheduler
- [VSC](https://vsc.ac.at) 4/5 (Staff, HPC), managed by VSC. access: univie
    * Our department has private Nodes on VSC4 and VSC5
    * Request an account with IT
    * slurm scheduler

---
class: center, middle
# How to connect? -  HOME / UNI

![](../img/IMGW-connection-diagram.png)
![](../img/IMGW-connection-diagram2.png)

---

# Tools - How to connect?
Depending on your operating system (OS) you should find the right tool to connect. 

Options:

- Web browser - Jupyterhub running Jupyter Notebooks
    * Teaching Hub on [SRVX1](https://srvx1.img.univie.ac.at/hub)
    * Research Hub on [JET01](https://wolke.img.univie.ac.at/researchhub)
- SSH (Windows: Putty, ...)
- SFTP (Windows: Mobaxterm, winscp, ...)
- VNC to SRVX8
    * install a vnc client on your OS
    * launch a vnc server on SRVX8, run: `userservices vnc -h` 
    * more information on @gitlab @VNC

---

# Login and Passwords 

When you login the first time, you will need to change your password. **Please be advised that you should choose a secure one.** *A password manager is recommended.*

```bash
$ ssh [user]@srvx1.img.univie.ac.at
$ ssh [user]@jet01.img.univie.ac.at
# from home to jet directly
$ ssh -J [user]@srvx1.img.univie.ac.at [user]@jet01.img.univie.ac.at
# or using the ZID server
$ ssh -J [u:account]@login.univie.ac.at [user]@jet01.img.univie.ac.at
```

Each of these require a different password:

- IMGW Servers (srvx1, srvx8, jet)
- u:account
- eduroam
- TeachingHub + OTP
- (optional) iLibrarian

---

# Services @ Servers

All servers try to be up to date and will be updated regularly, the base OS is Red Hat OS 8/AlmaLinux 8. 

All servers handle software like this:
- environment modules, run: `modules av`
    * different compilers: gnu, intel
    * libraries: math, eccodes, netcdf ,...
    * different MPI
    * tools: teleport, ecaccess, ...
- userservices, run: `userservices`
    * **quota**
    * server-schedule
    * weather
    * containers
    * yopass
    * filetransfer
    * transfersh
- singularity containers, advanced topic.
    * building singularity containers for HPC applications
    
---

# Services @ Servers 2

Locations on Servers are very important. Where to store and find what?

| name | Server | Path | Quota | comment |
| --- | --- | --- | --- | --- |
| HOME | srvx1 | `/users/staff/[user]` | 100 GB | source code, daily backup |
| HOME | jet01 | `/jetfs/home/[user]` | 100 GB | source code, daily backup |
| SCRATCH | srvx1 | `/scratch/das/[user]` | 1 TB | data, no backup |
| SCRATCH | srvx1 | `/scratch/mas/[user]` | 1 TB | data, no backup |
| SCRATCH | jet01 | `/jetfs/scratch/[user]` | no | data, no backup |
| DATA | srvx1 | `/data/[dataset]` | no | data, daily backup, constant |
| SHARED-DATA | srvx1 | `/scratch/shared/[dataset]` | no| data, no backup, shared |
| SHARED-DATA | jet01 | `/jetfs/shared-data/[dataset]` | no| data, no backup, shared |

All these paths can be remembered by running `userpaths`


---

# DATA Management

Science is about computing, but mostly about data and storaging that data.

| name | Server | Path |
| --- | --- | --- |
| DATA | srvx1 | `/data/[dataset]` | 
| SHARED-DATA | srvx1 | `/scratch/shared/[dataset]` |
| SHARED-DATA | jet01 | `/jetfs/shared-data/[dataset]` |

Please fill in the **Database** and follow the **Data Guidelines**, that can be found in these directories.
**Add a short decsription of your data and whom to contact for more information.**


---

# Support & Contacts

In case you need help, whom to contact?

- Servers > M. Blaschek
- VSC > M. Meindl / M. Blaschek
- Gitlab > M. Meindl / M. Blaschek
- Everything else > M. Ristic

Use Mattermost or send mail to [IT](mailto:it.img-wien@univie.ac.at) or [Gitlab Issues](https://gitlab.phaidra.org/imgw/computer-resources/-/issues)


---
class: center, title

# Questions

```
???????
?????
???
??
?
```
